/**
 * EasyBeans
 * Copyright (C) 2006 Bull S.A.S.
 * Contact: easybeans@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: EJB3DeploymentDesc.java 133 2006-03-06 18:00:57Z benoitf $
 * --------------------------------------------------------------------------
 */

package org.objectweb.easybeans.deployment.xml;

import java.io.File;
import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.jar.JarEntry;
import java.util.jar.JarFile;

import org.objectweb.easybeans.deployment.xml.parsing.EJB3DeploymentDescLoader;
import org.objectweb.easybeans.deployment.xml.parsing.ParsingException;
import org.objectweb.easybeans.deployment.xml.struct.EJB3;

/**
 * Utility class to get a deployment descriptor.
 * @author Florent Benoit
 */
public final class EJB3DeploymentDesc {

    /**
     * Utility class, no public constructor.
     */
    private EJB3DeploymentDesc() {

    }

    /**
     * Gets EJB3 for a given archive.
     * @param archive file/directory.
     * @throws EJB3DeploymentDescException if no xml is present.
     * @return EJB3 instance or null.
     */
    public static EJB3 getEjb3(final File archive) throws EJB3DeploymentDescException {
        // Parse XML if any
        URL ejbjarXmlURL = null;
        if (archive.isFile()) {
            JarFile jFile = null;
            JarEntry jarEntry = null;
            try {
                jFile = new JarFile(archive);
                jarEntry = jFile.getJarEntry("META-INF/ejb-jar.xml");
            } catch (IOException e) {
                throw new EJB3DeploymentDescException("Cannot get jarFile on file '" + archive + "'.");
            } finally {
                if (jFile != null) {
                    try {
                        jFile.close();
                    } catch (IOException e) {
                        throw new EJB3DeploymentDescException("Cannot close jarFile '" + jFile + "'.");
                    }
                }
            }
            if (jarEntry != null) {
                try {
                    ejbjarXmlURL = new URL("jar:" + archive.toURL() + "!/META-INF/ejb-jar.xml");
                } catch (MalformedURLException e) {
                    throw new EJB3DeploymentDescException("Cannot get jarFile on file '" + archive + "'.");
                }
            }

        } else {
            File f = new File(archive, "META-INF" + File.separator + "ejb-jar.xml");
            try {
                if (f.exists()) {
                    ejbjarXmlURL = f.toURL();
                }
            } catch (MalformedURLException e) {
                throw new EJB3DeploymentDescException("Invalid url", e);
            }
        }
        EJB3 ejb3 = null;
        // Get EJB3 object (if url valid)
        if (ejbjarXmlURL != null) {
             try {
                 ejb3 = EJB3DeploymentDescLoader.loadDeploymentDescriptor(ejbjarXmlURL);
            } catch (ParsingException e) {
                throw new EJB3DeploymentDescException("Cannot parse URL '" + ejbjarXmlURL + "'.");
            }
        }
        return ejb3;
    }
}
