/**
 * EasyBeans
 * Copyright (C) 2006 Bull S.A.S.
 * Contact: easybeans@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: AbsCallRef.java 882 2006-07-18 09:15:39Z benoitf $
 * --------------------------------------------------------------------------
 */

package org.objectweb.easybeans.rpc;

import javax.naming.NamingException;
import javax.naming.Reference;
import javax.naming.Referenceable;
import javax.naming.StringRefAddr;

/**
 * Define a common Referenceable objectd used by local or remote EJB.
 * @author Florent Benoit
 */
public abstract class AbsCallRef implements Referenceable {

    /**
     * Property used for referencing the container ID.
     */
    public static final String CONTAINER_ID = "containerID";

    /**
     * Property used for referencing the name of the factory.
     */
    public static final String FACTORY_NAME = "factoryName";

    /**
     * Property used for referencing the interface class name.
     */
    public static final String INTERFACE_NAME = "interfaceClassName";

    /**
     * Property used for using an unique ID or not.
     */
    public static final String USE_ID = "useID";

    /**
     * Name of the interface class.
     */
    private String itfClassName = null;

    /**
     * Container id.
     */
    private String containerId = null;

    /**
     * Factory name.
     */
    private String factoryName = null;

    /**
     * useID : true if all instance build with this ref are unique (stateful), false if it references the same object (stateless).
     */
    private boolean useID;

    /**
     * Constructor : build a reference.
     * @param itfClassName the name of the interface.
     * @param containerId the ID of the container.
     * @param factoryName the name of the factory
     * @param useID true if all instance build with this ref are unique
     *        (stateful), false if it references the same object (stateless)
     */
    public AbsCallRef(final String itfClassName, final String containerId, final String factoryName, final boolean useID) {
        this.itfClassName = itfClassName;
        this.containerId = containerId;
        this.factoryName = factoryName;
        this.useID = useID;
    }

    /**
     * Retrieves the Reference of this object.
     * @return The non-null Reference of this object.
     * @exception NamingException If a naming exception was encountered while
     *            retrieving the reference.
     */
    public abstract Reference getReference() throws NamingException;


    /**
     * Adds some settings to the reference.
     * @param reference the reference to configure
     * @return the updated reference.
     */
    protected Reference updateRefAddr(final Reference reference) {
        reference.add(new StringRefAddr(CONTAINER_ID, containerId));
        reference.add(new StringRefAddr(FACTORY_NAME, factoryName));
        reference.add(new StringRefAddr(INTERFACE_NAME, itfClassName));
        reference.add(new StringRefAddr(USE_ID, Boolean.toString(useID)));
        return reference;
    }

    /**
     * Gets the interface class name.
     * @return the name of the interface.
     */
    public String getItfClassName() {
        return itfClassName;
    }


}
